#!/usr/bin/perl
use strict;
#-----------------------
#para concatemers, gera tags e as estatisticas.
my @sizes;
my $minimum_size = 32;
my $maximum_size = 34;

my $input = "";
my $output = "";
my $routput = "";

my $version = 0.41;
my $vname = "extract_ditags.pl";

if (!$ARGV[0] || $ARGV[0]eq "-h") {
	print "\n\t-min <integer> \t:: set minimum size of the ditags [default:$minimum_size]\n",
	"\t-max <integer> \t:: set maximum size of the ditags [default:$maximum_size]\n",
	"\t-i <FILE> \t:: input file name [default:STDIN]\n",
	"\t-o <FILE> \t:: output file name [default:STDOUT]\n",
	"\t-r <FILE> \t:: report file name [default:STDERR]\n",
	"\t-h      \t:: display help information\n",
	"\t-v      \t:: display current version\n",
	"\n\t$vname Version $version\n\n";
	exit;
}

if ($ARGV[0] eq "-v") {
	print ("\n\t$vname Version $version\n\n");
	exit;
}
#
##first read command line to collect arguments
#
while (my $args = shift(@ARGV)) {
	#
	#minimun size
	#
	if ($args eq "-min") {
		if ($args = shift(@ARGV)) {
			$minimum_size = $args;
		}
		else {
			print STDERR "MINIMUM SIZE NOT SPECIFIED!\n";
			exit;
		}
	}

	#	
	#maximun size
	#
	elsif ($args eq "-max") {
		if ($args = shift(@ARGV)) {
			$maximum_size = $args;
		}
		else {
			print STDERR "MAXIMUM SIZE NOT SPECIFIED!\n";
			exit;
		}
	}

	#
	#input file
	#
	elsif ($args eq "-i") {
		if ($args = shift(@ARGV)) {
			$input = $args;
		}
		else {
			print STDERR "INPUT NOT SPECIFIED!\n";
			exit;
		}
	}
	#
	#output file
	#
	elsif ($args eq "-o") {
		if ($args = shift(@ARGV)) {
			$output = $args;
		}
		else {
			print STDERR "OUTPUT NOT SPECIFIED!\n";
			exit;
		}
	}
	#
	#output file
	#
	elsif ($args eq "-r") {
		if ($args = shift(@ARGV)) {
			$routput = $args;
		}
		else {
			print STDERR "REPORT OUTPUT NOT SPECIFIED!\n";
			exit;
		}
	}

}

if ($input) {
	open INPUT, "<$input" or die "COULD NOT OPEN $input :: $!\n";
	$input = "<INPUT>";
}
else {
	$input = "<>";
}

if ($output) {
	open OUTPUT, ">$output" or die "COULD NOT OPEN $output :: $!\n";
}

if ($routput) {
	open ROUTPUT, ">$routput" or die "COULD NOT OPEN $routput :: $!\n";
}



$/ = ">";
eval ($input);
while (my $entry = eval ($input)){
	chomp($entry);
	#use only lower case letters to make it simple
	$entry =~ tr/ATCG/atcg/;
	##
	###If you want multi-line fasta use teh code in this comment instead
	##
	##my @lines = split("\n", $entry);
	##my $name = shift(@lines);
	##my $concatamer = join(@lines);
	##
	(my $name, my $concatamer) = split("\n", $entry);
	process_concatamer($minimum_size,$maximum_size,$concatamer);

}

my $entries = @sizes;
my $accumulated_total = 0;
&printerr ("MAX SIZE=$entries\n");
for(my $i = 0; $i < $entries; $i++){
	if ($sizes[$i] != 0){
		$accumulated_total += $sizes[$i];
		&printerr ("size $i: $sizes[$i] entries(up to now:$accumulated_total)\n");
	}
}


sub process_concatamer {
	my $minimum_size = shift;
	my $maximum_size = shift;
	my $concatamer = shift;
	&printerr ("===CONCATAMER:\n$concatamer\n");
	my @all_ditags = split("catg",$concatamer);
	my $size = @all_ditags;
	my $current = 0;
	my $ditag = shift(@all_ditags);
	my $separator = "catg";
	my $comment = $ditag;


	while ($current < $size){
		my $size_ditag = length($ditag);
		if ( ($minimum_size <= $size_ditag) &&
			($size_ditag <= $maximum_size)){
			&printerr ("\tGOOD DITAG:$ditag\n");
			&printout ($ditag,"\n");
			$sizes[$size_ditag] += 1;
			$current += 1;
			$ditag = shift(@all_ditags);
			$comment = $ditag
		}
		elsif ($size_ditag > $maximum_size){
			process_long_ditags($minimum_size,$maximum_size,$ditag);
			$current += 1;
			$ditag = shift(@all_ditags);
			$comment = $ditag
		}	 
		else {

			#
			#ditag is too small
			#
			my $next_ditag = shift(@all_ditags);
			my $size_next_ditag = length($next_ditag);
			my $combined_size = $size_ditag + 4 + $size_next_ditag;
			if ((($minimum_size/2) <= ($size_ditag+3)) && 
				(($size_ditag+1) <= ($maximum_size/2)) &&
				(($minimum_size/2) <= ($size_next_ditag+3)) && 
				(($size_next_ditag+1) <= ($maximum_size/2)) &&
				($minimum_size <= $combined_size) && 
				($combined_size <= $maximum_size)){
				#
				#seems that we have catg in the middle of a ditag
				#
				$ditag .= "$separator"."$next_ditag";
				$comment .= "[catg]$next_ditag";
				$sizes[$combined_size] += 1;
				&printerr ("JOINED DITAG:$ditag($comment)");
				&printout ("$ditag\n");
				#
				#get next ditag
				#
				$ditag = shift(@all_ditags);
				$current += 2;
				$comment = $ditag;

			}
			else {
				#
				#we can not accept catg in the middle otherwise
				# discard ditag
				&printerr ("\tBAD DITAG(Small:$size_ditag,$size_next_ditag):$ditag($comment)\n");
				$sizes[$size_ditag] += 1;
				$ditag = $next_ditag;
				$current += 1;
				$comment = $ditag;
			}
		}
	}
}



sub process_long_ditags {
	my $minimum_size = shift;
	my $maximum_size = shift;
	my $large_ditag = shift;
	my $variations = "(aatg|tatg|gatg|cctg|cttg|cgtg|cacg|caag|cagg|catc|cata|catt)";

	my @all_fields = split(/$variations/,$large_ditag);
	my $size = @all_fields;
	my $current = 0;
	my $ditag = shift(@all_fields);
	my $separator = shift(@all_fields);
	my $comment = $ditag;


	LOOP:
	while ($current < $size){
		my $size_ditag = length($ditag);
		if ( ($minimum_size <= $size_ditag) &&
			($size_ditag <= $maximum_size)){
			&printerr ("\tJOINED DITAG:$ditag($comment)\n");
			&printout ($ditag,"\n");
			$sizes[$size_ditag] += 1;
			$current += 2;
			$ditag = shift(@all_fields);
			$separator = shift(@all_fields);
			$comment = $ditag;
		}
		elsif ($size_ditag > $maximum_size){
			#something is REALLY wrong!!
			&printerr ("\tBAD DITAG(Large:$size_ditag):$ditag($comment)\n");
			$sizes[$size_ditag] += 1;
			$current += 2;
			$ditag = shift(@all_fields);
			$separator = shift(@all_fields);
			$comment = $ditag;
		}	 
		else {

			#
			#ditag is too small
			#
			my $next_ditag = shift(@all_fields);
			my $next_separator = shift(@all_fields);
			my $size_next_ditag = length($next_ditag);
			#
			#separator is NOT "catg"
			#   in this case we keep joining ditags until correct size
			#
			$ditag .= $separator.$next_ditag;
			$comment .= "[$separator]$next_ditag";
			$separator = $next_separator;
			$current += 2;
		}

	}

}



sub printout {
	if ($output) {
		while (my $p = shift()) {
			print OUTPUT $p;
		}
	}
	else {
		while (my $p = shift()) {
			print $p;
		}
	}
}


sub printerr {
	if ($routput) {
		while (my $p = shift()) {
			print ROUTPUT $p;
		}
	}
	else {
		while (my $p = shift()) {
			print STDERR $p;
		}
	}
}
